@@ -16,21 +16,15 @@
 //! The EndBASIC language parser and interpreter.
 
 // Keep these in sync with other top-level files.
+#![allow(clippy::await_holding_refcell_ref)]
 #![warn(anonymous_parameters, bad_style, missing_docs)]
 #![warn(unused, unused_extern_crates, unused_import_braces, unused_qualifications)]
 #![warn(unsafe_code)]
 
 // TODO(jmmv): Should narrow the exposed interface by 1.0.0.
 pub mod ast;
-pub mod console;
-mod editor;
 pub mod eval;
 pub mod exec;
-pub mod help;
 mod lexer;
-pub mod numerics;
 mod parser;
-pub mod program;
 mod reader;
-pub mod repl;
-pub mod strings;
