@@ -15,8 +15,9 @@
 
 //! Statement and expression parser for the EndBASIC language.
 
-use crate::ast::{ArgSep, Expr, Statement, VarRef};
+use crate::ast::{ArgSep, Expr, Statement, VarRef, VarType};
 use crate::lexer::{Lexer, PeekableLexer, Token};
+use std::cmp::Ordering;
 use std::io;
 
 /// Parser errors.
@@ -45,6 +46,7 @@ pub type Result<T> = std::result::Result<T, Error>;
 #[derive(Debug, Eq, PartialEq)]
 enum ExprOp {
     LeftParen,
+    Call(VarRef),
 
     Add,
     Subtract,
@@ -93,6 +95,7 @@ impl ExprOp {
     fn priority(&self) -> i8 {
         match self {
             ExprOp::LeftParen => 5,
+            ExprOp::Call(_) => 5,
 
             ExprOp::Negate => 4,
             ExprOp::Not => 4,
@@ -158,6 +161,7 @@ impl ExprOp {
             ExprOp::Not => apply1(exprs, Expr::Not),
 
             ExprOp::LeftParen => Ok(()),
+            ExprOp::Call(_) => Ok(()),
         }
     }
 }
@@ -267,6 +271,7 @@ impl<'a> Parser<'a> {
     fn parse_expr(&mut self) -> Result<Option<Expr>> {
         let mut exprs: Vec<Expr> = vec![];
         let mut ops: Vec<ExprOp> = vec![];
+        let mut arity: Vec<usize> = vec![];
 
         let mut need_operand = true; // Also tracks whether an upcoming minus is unary.
         loop {
@@ -279,32 +284,97 @@ impl<'a> Parser<'a> {
                 Ok(())
             };
 
+            // Stop processing if we encounter an expression separator, but don't consume it because
+            // the caller needs to have access to it.
             match self.lexer.peek()? {
-                Token::Eof | Token::Eol | Token::Comma | Token::Semicolon | Token::Then => break,
+                Token::Eof
+                | Token::Eol
+                | Token::Semicolon
+                | Token::Then
+                | Token::To
+                | Token::Step => break,
+                Token::Comma => {
+                    // The comma is a special separator: if it's outside of a function call, the
+                    // expression is complete.  But if it is within a function call, we need to
+                    // continue our processing to extract function arguments.
+                    if arity.is_empty() {
+                        break;
+                    }
+                }
                 _ => (),
             };
+
             let token = self.lexer.consume_peeked();
             match token {
                 Token::Boolean(b) => handle_operand(Expr::Boolean(b))?,
+                Token::Double(d) => handle_operand(Expr::Double(d))?,
                 Token::Integer(i) => handle_operand(Expr::Integer(i))?,
                 Token::Text(t) => handle_operand(Expr::Text(t))?,
                 Token::Symbol(vref) => handle_operand(Expr::Symbol(vref))?,
 
                 Token::LeftParen => {
-                    if !need_operand {
-                        return Err(Error::Bad("Unexpected value in expression".to_owned()));
-                    }
-                    ops.push(ExprOp::LeftParen);
+                    // If the last operand we encountered was a symbol, collapse it and the left
+                    // parenthesis into the beginning of a function call.
+                    match exprs.pop() {
+                        Some(Expr::Symbol(vref)) => {
+                            debug_assert!(!need_operand);
+                            ops.push(ExprOp::Call(vref));
+                            arity.push(exprs.len());
+                        }
+                        e => {
+                            if let Some(e) = e {
+                                // We popped out the last expression to see if it this left
+                                // parenthesis started a function call... but if it didn't, we have
+                                // to put the expression back.
+                                exprs.push(e);
+                            }
+                            if !need_operand {
+                                return Err(Error::Bad(
+                                    "Unexpected value in expression".to_owned(),
+                                ));
+                            }
+                            ops.push(ExprOp::LeftParen);
+                        }
+                    };
                     need_operand = true;
                 }
                 Token::RightParen => {
                     while let Some(op) = ops.pop() {
                         op.apply(&mut exprs)?;
-                        if op == ExprOp::LeftParen {
+                        match op {
+                            ExprOp::LeftParen | ExprOp::Call(_) => {
+                                ops.push(op);
+                                break;
+                            }
+                            _ => (),
+                        };
+                    }
+
+                    match ops.pop() {
+                        Some(ExprOp::LeftParen) => {}
+                        Some(ExprOp::Call(name)) => {
+                            let n = exprs.len() - arity.pop().unwrap();
+                            let params = exprs.split_off(exprs.len() - n);
+                            exprs.push(Expr::Call(name, params));
+                        }
+                        Some(_) => (),
+                        None => return Err(Error::Bad("Unbalanced parenthesis".to_owned())),
+                    }
+                    need_operand = false;
+                }
+                Token::Comma => {
+                    debug_assert!(!ops.is_empty());
+                    if need_operand {
+                        return Err(Error::Bad("Missing expression after comma".to_owned()));
+                    }
+                    while let Some(op) = ops.pop() {
+                        op.apply(&mut exprs)?;
+                        if let ExprOp::Call(_) = op {
+                            ops.push(op);
                             break;
                         }
                     }
-                    need_operand = false;
+                    need_operand = true;
                 }
 
                 Token::Not => {
@@ -312,11 +382,22 @@ impl<'a> Parser<'a> {
                     need_operand = true;
                 }
                 Token::Minus => {
+                    let op;
                     if need_operand {
-                        ops.push(ExprOp::Negate);
+                        op = ExprOp::Negate;
                     } else {
-                        ops.push(ExprOp::Subtract);
+                        op = ExprOp::Subtract;
+                        while let Some(op2) = ops.last() {
+                            match *op2 {
+                                ExprOp::LeftParen | ExprOp::Call(_) => break,
+                                _ if op2.priority() < op.priority() => break,
+                                _ => (),
+                            };
+                            let op2 = ops.pop().unwrap();
+                            op2.apply(&mut exprs)?;
+                        }
                     }
+                    ops.push(op);
                     need_operand = true;
                 }
 
@@ -335,9 +416,11 @@ impl<'a> Parser<'a> {
                 | Token::Xor => {
                     let op = ExprOp::from(token);
                     while let Some(op2) = ops.last() {
-                        if *op2 == ExprOp::LeftParen || op2.priority() < op.priority() {
-                            break;
-                        }
+                        match *op2 {
+                            ExprOp::LeftParen | ExprOp::Call(_) => break,
+                            _ if op2.priority() < op.priority() => break,
+                            _ => (),
+                        };
                         let op2 = ops.pop().unwrap();
                         op2.apply(&mut exprs)?;
                     }
@@ -347,18 +430,32 @@ impl<'a> Parser<'a> {
 
                 Token::Bad(e) => return Err(Error::Bad(e)),
 
-                Token::Eof | Token::Eol | Token::Comma | Token::Semicolon | Token::Then => {
+                Token::Eof
+                | Token::Eol
+                | Token::Semicolon
+                | Token::Then
+                | Token::To
+                | Token::Step => {
                     panic!("Field separators handled above")
                 }
 
-                Token::If | Token::Else | Token::Elseif | Token::End | Token::While => {
+                Token::If
+                | Token::Else
+                | Token::Elseif
+                | Token::End
+                | Token::While
+                | Token::For
+                | Token::Next => {
                     return Err(Error::Bad("Unexpected keyword in expression".to_owned()));
                 }
             };
         }
 
         while let Some(op) = ops.pop() {
-            op.apply(&mut exprs)?;
+            match op {
+                ExprOp::LeftParen => return Err(Error::Bad("Unbalanced parenthesis".to_owned())),
+                _ => op.apply(&mut exprs)?,
+            }
         }
 
         if let Some(expr) = exprs.pop() {
@@ -437,6 +534,102 @@ impl<'a> Parser<'a> {
         self.reset()
     }
 
+    /// Extracts the optional `STEP` part of a `FOR` statement, with a default of 1.
+    fn parse_step(&mut self) -> Result<i32> {
+        match self.lexer.peek()? {
+            Token::Step => self.lexer.consume_peeked(),
+            _ => return Ok(1),
+        };
+
+        match self.lexer.peek()? {
+            Token::Integer(i) => {
+                let i = *i;
+                self.lexer.consume_peeked();
+                Ok(i)
+            }
+            Token::Minus => {
+                self.lexer.consume_peeked();
+                match self.lexer.peek()? {
+                    Token::Integer(i) => {
+                        let i = *i;
+                        self.lexer.consume_peeked();
+                        Ok(-i)
+                    }
+                    _ => Err(Error::Bad("STEP needs an integer".to_owned())),
+                }
+            }
+            _ => Err(Error::Bad("STEP needs an integer".to_owned())),
+        }
+    }
+
+    /// Parses a `FOR` statement.
+    fn parse_for(&mut self) -> Result<Statement> {
+        let iterator = match self.lexer.read()? {
+            Token::Symbol(iterator) => match iterator.ref_type() {
+                // TODO(jmmv): Should we support doubles in for loops?
+                VarType::Auto | VarType::Integer => iterator,
+                _ => {
+                    return Err(Error::Bad(
+                        "Iterator name in FOR statement must be an integer reference".to_owned(),
+                    ))
+                }
+            },
+            _ => return Err(Error::Bad("No iterator name in FOR statement".to_owned())),
+        };
+
+        self.expect_and_consume(Token::Equal, "No equal sign in FOR statement")?;
+        let start = match self.parse_expr()? {
+            Some(expr) => expr,
+            None => return Err(Error::Bad("No start expression in FOR statement".to_owned())),
+        };
+
+        self.expect_and_consume(Token::To, "No TO in FOR statement")?;
+        let end = match self.parse_expr()? {
+            Some(expr) => expr,
+            None => return Err(Error::Bad("No end expression in FOR statement".to_owned())),
+        };
+
+        let step = self.parse_step()?;
+        let end_condition = match step.cmp(&0) {
+            Ordering::Greater => {
+                Expr::LessEqual(Box::from(Expr::Symbol(iterator.clone())), Box::from(end))
+            }
+            Ordering::Less => {
+                Expr::GreaterEqual(Box::from(Expr::Symbol(iterator.clone())), Box::from(end))
+            }
+            Ordering::Equal => {
+                return Err(Error::Bad("Infinite FOR loop; STEP cannot be 0".to_owned()))
+            }
+        };
+
+        let next_value =
+            Expr::Add(Box::from(Expr::Symbol(iterator.clone())), Box::from(Expr::Integer(step)));
+
+        self.expect_and_consume(Token::Eol, "Expecting newline after FOR")?;
+
+        let stmts = self.parse_until(&[Token::Next])?;
+        self.expect_and_consume(Token::Next, "FOR without NEXT")?;
+
+        Ok(Statement::For(iterator, start, end_condition, next_value, stmts))
+    }
+
+    /// Advances until the next statement after failing to parse a `FOR` statement.
+    fn reset_for(&mut self) -> Result<()> {
+        loop {
+            match self.lexer.peek()? {
+                Token::Eof => break,
+                Token::Next => {
+                    self.lexer.consume_peeked();
+                    break;
+                }
+                _ => {
+                    self.lexer.consume_peeked();
+                }
+            }
+        }
+        self.reset()
+    }
+
     /// Parses a `WHILE` statement.
     fn parse_while(&mut self) -> Result<Statement> {
         let expr = match self.parse_expr()? {
@@ -494,6 +687,13 @@ impl<'a> Parser<'a> {
                 }
                 Ok(Some(result?))
             }
+            Token::For => {
+                let result = self.parse_for();
+                if result.is_err() {
+                    self.reset_for()?;
+                }
+                Ok(Some(result?))
+            }
             Token::Symbol(vref) => {
                 let peeked = self.lexer.peek()?;
                 if *peeked == Token::Equal {
@@ -792,18 +992,88 @@ mod tests {
     }
 
     #[test]
-    fn test_expr_integer_signs() {
+    fn test_expr_numeric_signs() {
         use Expr::*;
+
         do_expr_ok_test("-a", Negate(Box::from(Symbol(VarRef::new("a", VarType::Auto)))));
+
         do_expr_ok_test(
             "1 - -3",
             Subtract(Box::from(Integer(1)), Box::from(Negate(Box::from(Integer(3))))),
         );
+        do_expr_ok_test(
+            "-1 - 3",
+            Subtract(Box::from(Negate(Box::from(Integer(1)))), Box::from(Integer(3))),
+        );
         do_expr_ok_test(
             "5 + -1",
             Add(Box::from(Integer(5)), Box::from(Negate(Box::from(Integer(1))))),
         );
+        do_expr_ok_test(
+            "-5 + 1",
+            Add(Box::from(Negate(Box::from(Integer(5)))), Box::from(Integer(1))),
+        );
         do_expr_ok_test("NOT -3", Not(Box::from(Negate(Box::from(Integer(3))))));
+
+        do_expr_ok_test(
+            "1.0 - -3.5",
+            Subtract(Box::from(Double(1.0)), Box::from(Negate(Box::from(Double(3.5))))),
+        );
+        do_expr_ok_test(
+            "5.12 + -0.50",
+            Add(Box::from(Double(5.12)), Box::from(Negate(Box::from(Double(0.50))))),
+        );
+        do_expr_ok_test("NOT -3", Not(Box::from(Negate(Box::from(Integer(3))))));
+    }
+
+    #[test]
+    fn test_expr_functions_variadic() {
+        use Expr::*;
+        do_expr_ok_test("zero()", Call(VarRef::new("zero", VarType::Auto), vec![]));
+        do_expr_ok_test("one%(1)", Call(VarRef::new("one", VarType::Integer), vec![Integer(1)]));
+        do_expr_ok_test(
+            "many$(3, \"x\", TRUE)",
+            Call(
+                VarRef::new("many", VarType::Text),
+                vec![Integer(3), Text("x".to_owned()), Boolean(true)],
+            ),
+        );
+    }
+
+    #[test]
+    fn test_expr_functions_nested() {
+        use Expr::*;
+        do_expr_ok_test(
+            "outer?(1, inner1(2, 3), 4, inner2(), 5)",
+            Call(
+                VarRef::new("outer", VarType::Boolean),
+                vec![
+                    Integer(1),
+                    Call(VarRef::new("inner1", VarType::Auto), vec![Integer(2), Integer(3)]),
+                    Integer(4),
+                    Call(VarRef::new("inner2", VarType::Auto), vec![]),
+                    Integer(5),
+                ],
+            ),
+        );
+    }
+
+    #[test]
+    fn test_expr_functions_and_ops() {
+        use Expr::*;
+        do_expr_ok_test(
+            "b AND ask?(34 + 15, ask(1, FALSE))",
+            And(
+                Box::from(Symbol(VarRef::new("b".to_owned(), VarType::Auto))),
+                Box::from(Call(
+                    VarRef::new("ask", VarType::Boolean),
+                    vec![
+                        Add(Box::from(Integer(34)), Box::from(Integer(15))),
+                        Call(VarRef::new("ask", VarType::Auto), vec![Integer(1), Boolean(false)]),
+                    ],
+                )),
+            ),
+        );
     }
 
     #[test]
@@ -814,6 +1084,20 @@ mod tests {
         do_expr_error_test("((3)2)", "Unexpected value in expression");
         do_expr_error_test("2 3", "Unexpected value in expression");
 
+        do_expr_error_test("(", "Unbalanced parenthesis");
+        do_expr_error_test(")", "Unbalanced parenthesis");
+        do_expr_error_test("(()", "Unbalanced parenthesis");
+        do_expr_error_test("())", "Unbalanced parenthesis");
+        do_expr_error_test("3 + (2 + 1) + (4 - 5", "Unbalanced parenthesis");
+        do_expr_error_test("3 + 2 + 1) + (4 - 5)", "Unbalanced parenthesis");
+
+        do_expr_error_test("foo(,)", "Missing expression after comma");
+        do_expr_error_test("foo(3, , 4)", "Missing expression after comma");
+        // TODO(jmmv): These are not the best error messages...
+        do_expr_error_test("(,)", "Unbalanced parenthesis");
+        do_expr_error_test("(3, 4)", "Unbalanced parenthesis");
+        do_expr_error_test("((), ())", "Unbalanced parenthesis");
+
         // TODO(jmmv): This succeeds because `PRINT` is interned as a `Token::Symbol` so the
         // expression parser sees it as a variable reference... but this should probably fail.
         // Would need to intern builtin call names as a separate token to catch this, but that
@@ -827,7 +1111,7 @@ mod tests {
 
     #[test]
     fn test_expr_errors_due_to_keywords() {
-        for kw in &["IF", "ELSEIF", "ELSE", "END", "WHILE"] {
+        for kw in &["IF", "ELSEIF", "ELSE", "END", "WHILE", "FOR", "NEXT"] {
             do_expr_error_test(&format!("2 + {} - 1", kw), "Unexpected keyword in expression");
         }
     }
@@ -1002,6 +1286,117 @@ mod tests {
         do_error_test_no_reset("ELSE 1\nEND IF", "Unexpected token Else in statement");
     }
 
+    #[test]
+    fn test_for_empty() {
+        let auto_iter = VarRef::new("i", VarType::Auto);
+        do_ok_test(
+            "FOR i = 1 TO 10\nNEXT",
+            &[Statement::For(
+                auto_iter.clone(),
+                Expr::Integer(1),
+                Expr::LessEqual(
+                    Box::from(Expr::Symbol(auto_iter.clone())),
+                    Box::from(Expr::Integer(10)),
+                ),
+                Expr::Add(Box::from(Expr::Symbol(auto_iter)), Box::from(Expr::Integer(1))),
+                vec![],
+            )],
+        );
+
+        let typed_iter = VarRef::new("i", VarType::Integer);
+        do_ok_test(
+            "FOR i% = 1 TO 10\nREM Nothing to do\nNEXT",
+            &[Statement::For(
+                typed_iter.clone(),
+                Expr::Integer(1),
+                Expr::LessEqual(
+                    Box::from(Expr::Symbol(typed_iter.clone())),
+                    Box::from(Expr::Integer(10)),
+                ),
+                Expr::Add(Box::from(Expr::Symbol(typed_iter)), Box::from(Expr::Integer(1))),
+                vec![],
+            )],
+        );
+    }
+
+    #[test]
+    fn test_for_incrementing() {
+        let iter = VarRef::new("i", VarType::Auto);
+        do_ok_test(
+            "FOR i = 0 TO 5\nA\nB\nNEXT",
+            &[Statement::For(
+                iter.clone(),
+                Expr::Integer(0),
+                Expr::LessEqual(Box::from(Expr::Symbol(iter.clone())), Box::from(Expr::Integer(5))),
+                Expr::Add(Box::from(Expr::Symbol(iter)), Box::from(Expr::Integer(1))),
+                vec![
+                    Statement::BuiltinCall("A".to_owned(), vec![]),
+                    Statement::BuiltinCall("B".to_owned(), vec![]),
+                ],
+            )],
+        );
+    }
+
+    #[test]
+    fn test_for_incrementing_with_step() {
+        let iter = VarRef::new("i", VarType::Auto);
+        do_ok_test(
+            "FOR i = 0 TO 5 STEP 2\nA\nNEXT",
+            &[Statement::For(
+                iter.clone(),
+                Expr::Integer(0),
+                Expr::LessEqual(Box::from(Expr::Symbol(iter.clone())), Box::from(Expr::Integer(5))),
+                Expr::Add(Box::from(Expr::Symbol(iter)), Box::from(Expr::Integer(2))),
+                vec![Statement::BuiltinCall("A".to_owned(), vec![])],
+            )],
+        );
+    }
+
+    #[test]
+    fn test_for_decrementing_with_step() {
+        let iter = VarRef::new("i", VarType::Auto);
+        do_ok_test(
+            "FOR i = 5 TO 0 STEP -1\nA\nNEXT",
+            &[Statement::For(
+                iter.clone(),
+                Expr::Integer(5),
+                Expr::GreaterEqual(
+                    Box::from(Expr::Symbol(iter.clone())),
+                    Box::from(Expr::Integer(0)),
+                ),
+                Expr::Add(Box::from(Expr::Symbol(iter)), Box::from(Expr::Integer(-1))),
+                vec![Statement::BuiltinCall("A".to_owned(), vec![])],
+            )],
+        );
+    }
+
+    #[test]
+    fn test_for_errors() {
+        do_error_test("FOR\n", "No iterator name in FOR statement");
+        do_error_test("FOR =\n", "No iterator name in FOR statement");
+        do_error_test("FOR a$\n", "Iterator name in FOR statement must be an integer reference");
+        do_error_test("FOR d#\n", "Iterator name in FOR statement must be an integer reference");
+
+        do_error_test("FOR i 3\n", "No equal sign in FOR statement");
+        do_error_test("FOR i = TO\n", "No start expression in FOR statement");
+        do_error_test("FOR i = NEXT\n", "Unexpected keyword in expression");
+
+        do_error_test("FOR i = 3 STEP\n", "No TO in FOR statement");
+        do_error_test("FOR i = 3 TO STEP\n", "No end expression in FOR statement");
+        do_error_test("FOR i = 3 TO NEXT\n", "Unexpected keyword in expression");
+
+        do_error_test("FOR i = 3 TO 1 STEP a\n", "STEP needs an integer");
+        do_error_test("FOR i = 3 TO 1 STEP -a\n", "STEP needs an integer");
+        do_error_test("FOR i = 3 TO 1 STEP NEXT\n", "STEP needs an integer");
+        do_error_test("FOR i = 3 TO 1 STEP 0\n", "Infinite FOR loop; STEP cannot be 0");
+
+        do_error_test("FOR i = 3 TO 1", "Expecting newline after FOR");
+        do_error_test("FOR i = 1 TO 3 STEP 1", "Expecting newline after FOR");
+        do_error_test("FOR i = 3 TO 1 STEP -1", "Expecting newline after FOR");
+
+        do_error_test("FOR i = 0 TO 10\nPRINT i\n", "FOR without NEXT");
+    }
+
     #[test]
     fn test_while_empty() {
         do_ok_test(
